//
// "$Id: Fl_Counter_X.cxx 9637 2012-07-24 04:37:22Z matt $"
//
// Counter widget for the Fast Light Tool Kit (FLTK).
//
// Copyright 1998-2010 by Bill Spitzak and others.
//
// This library is free software. Distribution and use rights are outlined in
// the file "COPYING" which should have been included with this file.  If this
// file is missing or damaged, see the license at:
//
//     http://www.fltk.org/COPYING.php
//
// Please report all bugs and problems on the following page:
//
//     http://www.fltk.org/str.php
//

#include <FL/Fl.H>
#include <FL/Fl_Counter_X.H>
#include <FL/Fl_Simple_Counter.H>
#include <FL/fl_draw.H>
#include <Fl/xcounter_arrows.h>
#include <Fl/Fl_RGB_Image.h>

#define BVM_DOUBLE_ARROW_WIDTH 14
#define BVM_SINGLE_ARROW_WIDTH 10

//extern Fl_RGB_Image image_xfast_arrow_left;
extern Fl_RGB_Image image_xmedium_arrow_left;
extern Fl_RGB_Image image_xmedium_arrow_double_left;
//extern Fl_RGB_Image image_xslow_arrow_left;
//extern Fl_RGB_Image image_xslow_arrow_right;
extern Fl_RGB_Image image_xmeduim_arrow_right;
extern Fl_RGB_Image image_xmedium_arrow_double_right;
//extern Fl_RGB_Image image_xfast_arrow_right;

void Fl_Counter_X::ComputeSizes()
{
   if (type() == FL_NORMAL_COUNTER) {

    // <<
    xx[1] = x();	 ww[1] = BVM_DOUBLE_ARROW_WIDTH;
    // <
    xx[2] = x()+BVM_DOUBLE_ARROW_WIDTH;     ww[2] = BVM_SINGLE_ARROW_WIDTH;
    // output
    xx[0] = x()+BVM_DOUBLE_ARROW_WIDTH+BVM_SINGLE_ARROW_WIDTH;     ww[0] = w()-((BVM_DOUBLE_ARROW_WIDTH*2)+(BVM_SINGLE_ARROW_WIDTH*2));
    // >
    xx[3] = x()+BVM_DOUBLE_ARROW_WIDTH+BVM_SINGLE_ARROW_WIDTH + ww[0]; ww[3] = BVM_SINGLE_ARROW_WIDTH;
    // >>
    xx[4] = x()+BVM_DOUBLE_ARROW_WIDTH+BVM_SINGLE_ARROW_WIDTH + ww[0] + BVM_SINGLE_ARROW_WIDTH; ww[4] = BVM_DOUBLE_ARROW_WIDTH;
  } else {

    // unused
    xx[1] = 0;	         ww[1] = 0;
    // <
    xx[2] = x();	 ww[2] = BVM_SINGLE_ARROW_WIDTH;
    // output
    xx[0] = x()+BVM_SINGLE_ARROW_WIDTH;	 ww[0] = w()-(2*BVM_SINGLE_ARROW_WIDTH);
    // >
    xx[3] = x()+BVM_SINGLE_ARROW_WIDTH + ww[0]; ww[3] = BVM_SINGLE_ARROW_WIDTH;
    // unused
    xx[4] = 0;	         ww[4] = 0;
  }
}
 
void Fl_Counter_X::draw() {
  int i; Fl_Boxtype boxtype[5];
  Fl_Color selcolor;

  boxtype[0] = box();
  if (boxtype[0] == FL_UP_BOX) boxtype[0] = FL_DOWN_BOX;
  if (boxtype[0] == FL_THIN_UP_BOX) boxtype[0] = FL_THIN_DOWN_BOX;
  for (i=1; i<5; i++)
    if (mouseobj == i)
      boxtype[i] = fl_down(box());
    else
      boxtype[i] = box();

  ComputeSizes();
  

  draw_box(boxtype[0], xx[0], y(), ww[0], h(), FL_BACKGROUND2_COLOR);
  fl_font(textfont(), textsize());
  fl_color(active_r() ? textcolor() : fl_inactive(textcolor()));
  char str[128]; format(str);
  fl_draw(str, xx[0], y(), ww[0], h(), FL_ALIGN_CENTER);
  if (Fl::focus() == this) draw_focus(boxtype[0], xx[0], y(), ww[0], h());
  if (!(damage()&FL_DAMAGE_ALL)) return; // only need to redraw text

  if (active_r())
    selcolor = labelcolor();
  else
    selcolor = fl_inactive(labelcolor());

  if (type() == FL_NORMAL_COUNTER) {
    draw_box(boxtype[1], xx[1], y(), ww[1], h(), color());
    //fl_draw_symbol("@-4<<", xx[1], y(), ww[1], h(), selcolor);
    image_xmedium_arrow_double_left.draw(xx[1]+1, y(), ww[1], h());
  }
  draw_box(boxtype[2], xx[2], y(), ww[2], h(), color());
  //fl_draw_symbol("@-4<",  xx[2], y(), ww[2], h(), selcolor);
  image_xmedium_arrow_left.draw(xx[2]-1, y(), ww[2], h());
  draw_box(boxtype[3], xx[3], y(), ww[3], h(), color());
  //fl_draw_symbol("@-4>",  xx[3], y(), ww[3], h(), selcolor);
  image_xmedium_arrow_right.draw(xx[3]-1, y(), ww[3], h());
  if (type() == FL_NORMAL_COUNTER) {
    draw_box(boxtype[4], xx[4], y(), ww[4], h(), color());
    //fl_draw_symbol("@-4>>", xx[4], y(), ww[4], h(), selcolor);
    image_xmedium_arrow_double_right.draw(xx[4]+1, y(), ww[4], h());
  }
}

void Fl_Counter_X::increment_cb() {
  if (!mouseobj) return;
  double v = value();
  switch (mouseobj) {
  case 1: v -= lstep_; break;
  case 2: v = increment(v, -1); break;
  case 3: v = increment(v, 1); break;
  case 4: v += lstep_; break;
  }
  handle_drag(clamp(round(v)));
}

#define INITIALREPEAT .5
#define REPEAT .1

void Fl_Counter_X::repeat_callback(void* v) {
  Fl_Counter_X* b = (Fl_Counter_X*)v;
  if (b->mouseobj) {
    Fl::add_timeout(REPEAT, repeat_callback, b);
    b->increment_cb();
  }
}

int Fl_Counter_X::calc_mouseobj() {
   
  ComputeSizes();
  if (type() == FL_NORMAL_COUNTER) {
    if (Fl::event_inside(xx[1], y(), ww[1], h())) return 1;
    if (Fl::event_inside(xx[2], y(), ww[2], h())) return 2;
    if (Fl::event_inside(xx[3], y(), ww[3], h())) return 3;
    if (Fl::event_inside(xx[4], y(), ww[4], h())) return 4;
  } else {
    if (Fl::event_inside(xx[2], y(), ww[2], h())) return 2;
    if (Fl::event_inside(xx[3], y(), ww[3], h())) return 3;
  }
  return -1;
}

int Fl_Counter_X::handle(int event) {
  int i;
  switch (event) {
  case FL_RELEASE:
    if (mouseobj) {
      Fl::remove_timeout(repeat_callback, this);
      mouseobj = 0;
      redraw();
    }
    handle_release();
    return 1;
  case FL_PUSH:
    if (Fl::visible_focus()) Fl::focus(this);
    { Fl_Widget_Tracker wp(this);
      handle_push();
      if (wp.deleted()) return 1;
    }
  case FL_DRAG:
    i = calc_mouseobj();
    if (i != mouseobj) {
      Fl::remove_timeout(repeat_callback, this);
      mouseobj = (uchar)i;
      if (i) Fl::add_timeout(INITIALREPEAT, repeat_callback, this);
      Fl_Widget_Tracker wp(this);
      increment_cb();
      if (wp.deleted()) return 1;
      redraw();
    }
    return 1;
  case FL_KEYBOARD :
    switch (Fl::event_key()) {
      case FL_Left:
	handle_drag(clamp(increment(value(),-1)));
	return 1;
      case FL_Right:
	handle_drag(clamp(increment(value(),1)));
	return 1;
      default:
        return 0;
    }
    // break not required because of switch...
  case FL_FOCUS : /* FALLTHROUGH */
  case FL_UNFOCUS :
    if (Fl::visible_focus()) {
      redraw();
      return 1;
    } else return 0;
  case FL_ENTER : /* FALLTHROUGH */
  case FL_LEAVE :
    return 1;
  default:
    return 0;
  }
}

/**
  Destroys the valuator.
 */
Fl_Counter_X::~Fl_Counter_X() {
  Fl::remove_timeout(repeat_callback, this);
}

/**
  Creates a new Fl_Counter_X widget using the given position, size, and label
  string. The default type is FL_NORMAL_COUNTER.
  \param[in] X, Y, W, H position and size of the widget
  \param[in] L widget label, default is no label
 */
Fl_Counter_X::Fl_Counter_X(int X, int Y, int W, int H, const char* L)
  : Fl_Valuator(X, Y, W, H, L) {
  box(FL_UP_BOX);
  selection_color(FL_INACTIVE_COLOR); // was FL_BLUE
  //align(FL_ALIGN_BOTTOM);
  bounds(-1000000.0, 1000000.0);
  Fl_Valuator::step(1, 10);
  lstep_ = 1.0;
  mouseobj = 0;
  textfont_ = FL_SANS_SERIF;
  textsize_ = FL_NORMAL_SIZE;
  textcolor_ = FL_FOREGROUND_COLOR;
}

/*
Fl_Simple_Counter::Fl_Simple_Counter(int X,int Y,int W,int H, const char *L)
: Fl_Counter(X,Y,W,H,L) {
  type(FL_SIMPLE_COUNTER);
}
*/


//
// End of "$Id: Fl_Counter_X.cxx 9637 2012-07-24 04:37:22Z matt $".
//
